@@ -15,8 +15,6 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: dnssectool.c,v 1.63 2011/10/21 03:55:33 marka Exp $ */
-
 /*! \file */
 
 /*%
@@ -121,6 +119,12 @@ vbprintf(int level, const char *fmt, ...) {
 	va_end(ap);
 }
 
+void
+version(const char *program) {
+	fprintf(stderr, "%s %s\n", program, VERSION);
+	exit(0);
+}
+
 void
 type_format(const dns_rdatatype_t type, char *cp, unsigned int size) {
 	isc_buffer_t b;
